//------------------------------------------------------------------
// $Workfile: TaUtil_serialCom.h $
// $Archive: /eib_stk/appl/A1008TA_KIMaip_Eval/INC/TaUtil_serialCom.h $
//
// Project:           common utilities
//
// Description:
//
// $Author: Thiere_m $
//
// Copyright Tapko Technologies GmbH
//
// $Date: 20.05.11 10:54 $
// $Revision: 1 $
//------------------------------------------------------------------
#ifndef taUtil_serialCom_h
#define taUtil_serialCom_h

#include "TaUtil_types.h"  
#include "taUtil_fifo.h"

#define LSC_MAX_RX_LINE_LENGTH 80

#define LSC_UART0  0x00
#define LSC_UART1  0x01
#define LSC_UART2  0x02
#define LSC_UART3  0x03
#define LSC_UART4  0x04
#define LSC_UART5  0x05
#define LSC_UART6  0x06
#define LSC_UART7  0x07


#define LSC_ENABLE_UART0  0x01
#define LSC_ENABLE_UART1  0x02
#define LSC_ENABLE_UART2  0x04
#define LSC_ENABLE_UART3  0x08
#define LSC_ENABLE_UART4  0x10
#define LSC_ENABLE_UART5  0x20
#define LSC_ENABLE_UART6  0x40
#define LSC_ENABLE_UART7  0x80

#define LSC_TX_BLOCKING  0x01
#define LSC_RX_BLOCKING  0x02

#ifdef LSC_EnableTxLED
  #define LSC_SetTxLED()          APPL_SetTxLED()
#else
  #define LSC_SetTxLED()
#endif

#ifdef LSC_EnableRxLED
  #define LSC_SetRxLED()          APPL_SetRxLED()
#else
  #define LSC_SetRxLED()
#endif


enum LSC_data
{
  LSC_data7 = 0,
  LSC_data8 = 1
};

enum LSC_parity
{
  LSC_parityNone = 0,
  LSC_parityEven = 1,
  LSC_parityOdd = 2
};

enum LSC_stop
{
  LSC_stop1 = 0,
  LSC_stop2 = 1
};

enum LSC_baudrate
{
  LSC_baud1200   = 0,
  LSC_baud2400   = 1,
  LSC_baud4800   = 2,
  LSC_baud9600   = 3,
  LSC_baud19200  = 4,
  LSC_baud38400  = 5,
  LSC_baud57600  = 6,
  LSC_baud115200 = 7,
  LSC_baud230400 = 8,
  LSC_baud460800 = 9
};

typedef struct
{
  unsigned short portNr   : 3;
  unsigned short dataBits : 1;
  unsigned short parity   : 2;
  unsigned short stop     : 1;
  unsigned short dummy    : 1;
  unsigned short baudrate : 4;
  unsigned short flags;

  unsigned rxBufferSize;
  char *rxBuffer;
  unsigned txBufferSize;
  char *txBuffer;
} LSC_SerialConfig;

typedef struct
{
  unsigned short flags;

  unsigned short rxOverflow   : 1;
  unsigned short transmitting : 1;

  FIFO_Sstate txFifo;
  FIFO_Sstate rxFifo;
} LSC_SerialData;

extern LSC_SerialData *LSC_struct[LSC_noOfUarts];

void LSC_Init(void);
void LSC_ConfigUart(const LSC_SerialConfig *config, LSC_SerialData *settings);

void LSC_HWConfigUart(const LSC_SerialConfig *settings);
void LSC_HWTxByte(unsigned char portNr, unsigned char byte);

void LSC_TxInt(unsigned char portNr);
void LSC_RxInt(unsigned char portNr, unsigned char byte);
_BOOL LSC_TxByte(unsigned char portNr, unsigned char byte, unsigned char flags);
_BOOL LSC_RxByte(unsigned char portNr, char *byte, unsigned char flags);
_BOOL LSC_TxString(unsigned char portNr, char *data, unsigned char size, unsigned char flags);
_BOOL LSC_RxString(unsigned char portNr, char *data, unsigned char *size, unsigned char delimiter, unsigned char flags);
_BOOL LSC_TxSpaceFree(unsigned char portNr, unsigned bytes2Write);
void APPL_SetTxLED(void);
void APPL_SetRxLED(void);

_BOOL LSC_RxLine(unsigned char portNr, char *data);

#endif // taUtil_serialCom_h
