//------------------------------------------------------------------
//
// Copyright (c) 1999 - 2009 Tapko Technologies GmbH
//
// PROPRIETARY RIGHTS of Tapko Technologies GmbH are involved
// in the subject matter of this material. All manufacturing,
// reproduction, use, and sales rights pertaining to this subject matter
// are governed by the license agreement. The recipient of this software
// implicitly accepts the terms of the license.
//
// This file is part of the Libraries of Tapko Technologies GmbH
// (e.g. TAPKO EIB/KNX Communication Stack)
//
//------------------------------------------------------------------
// $Workfile: knx_target.h $   
// $Archive: /eib_stk/appl/A1008TA_KIMaip_Eval/INC/knx_target.h $     

// $Author: Thiere_m $         
// 
// $Date: 20.05.11 10:54 $
// $Revision: 1 $         
//------------------------------------------------------------------
// 21.05.2003 Macro WDG_STOP introduced
// 22.05.2003 UT_CopyNvMem, UT_FillNvMem changed to macro
// 16.03.2006 different files for compiler versions eleminated
//            differences are now solved via predefined switches
//------------------------------------------------------------------
#ifdef __IAR_SYSTEMS_ICC
#define COMPILE_C
#endif
#ifdef __TI_COMPILER_VERSION__
#define COMPILE_C
#endif

#include "KnxDev_buildType.h"
#include <Msp430_const.h>

//------------------------------------------------------------------
#ifdef COMPILE_C               // include only for C files
#ifdef KNX_BUILD_SYSTEM
//#include  <../ll/tpuart/tpuart.h>
#endif
void hwinit(void);
#endif

//------------------------------------------------------------------
// definitions for the target system
//------------------------------------------------------------------
#define TARGET_PLATFORM_STR            "Texas Instruments MSP430"
#define COMPILER_STR                   "IAR C-Compiler"
#define KAI_ENDIAN_FORMAT              KAI_LITTLE_ENDIAN

#define TARGET_OS            TARGET_OS_NONE
//------------------------------------------------------------------
// Interrupts
//------------------------------------------------------------------
#if defined(__IAR_SYSTEMS_ICC__) && __VER__ < 200
  #define _CRITICAL                      monitor
#endif
#if defined(__IAR_SYSTEMS_ICC__) && __VER__ >= 210
  #define _CRITICAL                      __monitor
#endif
#if defined(__IAR_SYSTEMS_ICC__) && __VER__ < 300
  #define inline
#endif
#ifdef __TI_COMPILER_VERSION__
  #define _CRITICAL                      // todo
#endif

#define MS_semaphore                  
#define MS_InitSemaphore()         
#define MS_EnterCriticalFunction()
#define MS_LeaveCriticalFunction()


//------------------------------------------------------------------
// initialization functions
//------------------------------------------------------------------
#ifdef COMPILE_C               // include only for C files
void MSP430F1xx_InitOsc(void);
void MSP430F14x_InitOsc(void);
void MSP430F54x_InitOsc(void);
void MSP430_InitTimer(void);
void MSP430F_InitUsart(void);
void MSP430F_InitUsci(void);
#endif
//------------------------------------------------------------------
// Watchdog
//------------------------------------------------------------------
#ifndef WDG_RESET

#if defined(MSP430F1xx) || defined(MSP430F2xx) 
#define WDG_RESET()         WDTCTL = WDTPW | WDTCNTCL;            // reset watchdog
#define WDG_START()         WDTCTL = WDTPW | WDTCNTCL;
#define WDG_STOP()          WDTCTL = WDTPW | WDTHOLD ;
#endif 

#if defined(MSP430F4xx)
#define WDG_RESET()         WDTCTL = WDTSSEL | WDTPW | WDTCNTCL;            // reset watchdog
#define WDG_START()         WDTCTL = WDTSSEL | WDTPW | WDTCNTCL;
#define WDG_STOP()          WDTCTL = WDTSSEL | WDTPW | WDTHOLD ;
#endif 

#if defined(MSP430F54xx)
#define WDG_RESET()         WDTCTL = WDTSSEL1 | WDTPW | WDTCNTCL;            // reset watchdog
#define WDG_START()         WDTCTL = WDTSSEL1 | WDTPW | WDTCNTCL;
#define WDG_STOP()          WDTCTL = WDTSSEL1 | WDTPW | WDTHOLD ;
#endif
#endif

//------------------------------------------------------------------
#ifndef MSP_FLASH_ERASE_START
#define MSP_FLASH_ERASE_START() WDG_STOP();
#define MSP_FLASH_ERASE_END()   WDG_RESET(); 
#define MSP_FLASH_WRITE_START() WDG_RESET();
#define MSP_FLASH_WRITE_END()   

#define MSP_WaitFlashWriteAllowed()
#endif

//------------------------------------------------------------------
// defininitions for target EEPROM
//------------------------------------------------------------------
#define TARGET_EEPROM     TARGET_EEPROM_READ_NORMAL

//------------------------------------------------------------------
// definitions for flash write routines
//------------------------------------------------------------------
#ifdef COMPILE_C                                   // only for C files
extern unsigned char *MSP_source;
extern unsigned char *MSP_dest;
extern unsigned short MSP_count;
void MSP_WriteBlock(unsigned char mode);
#define UT_CopyNvMem(source, dest, count)    {MSP_source = source; \
                                              MSP_dest = dest; \
                                              MSP_count = count; \
                                              MSP_WriteBlock(0);}

#define UT_FillNvMem(value, dest, count)     {MSP_source = &value; \
                                              MSP_dest = dest; \
                                              MSP_count = count; \
                                              MSP_WriteBlock(1);}

#ifndef ONLY_EEPROM                     // ETS project
extern unsigned char *MSP_dest;
#endif
_CRITICAL void MSP_FlashErase(void *ptr);
_CRITICAL void MSP_FlashWriteByte(char byte);
#endif

//------------------------------------------------------------------
// defininitions for the memory segments
//------------------------------------------------------------------
#if defined(__IAR_SYSTEMS_ICC__) && __VER__ < 200
  #define KNX_CONST    const
  #define KNX_NO_INIT  
#endif
#if defined(__IAR_SYSTEMS_ICC__) && __VER__ >= 210
  #define KNX_CONST    __root const
  #define KNX_NO_INIT  __no_init
#endif
#ifdef __TI_COMPILER_VERSION__
  #define KNX_CONST    const                    // todo
  #define KNX_NO_INIT  
#endif

#define MEMSEG_DUMMY
#define MEMSEG_BCURAM    KNX_NO_INIT
#define MEMSEG_FLASH_TMP KNX_CONST

#ifdef ONLY_EEPROM                     // ETS project
  #undef TARGET_EEPROM
  #define TARGET_EEPROM TARGET_EEPROM_READ_NORMAL
  #define ETS_DECLARE_PARAMETER
#endif

//------------------------------------------------------------------
// definitions of registers of MSP430
//------------------------------------------------------------------
#ifndef SELM_DCOCLK
#define SELM_DCOCLK     0
#endif
#ifndef SELM_XT2CLK
#define SELM_XT2CLK     SELM1
#endif
#define SELM_XLFT1CLK   SELM0 | SELM1

#ifndef SELS_DCOCLK
#define SELS_DCOCLK     0
#define SELS_XT2CLK     SELS
#endif

#ifndef TASSEL_TACLK
#define TASSEL_TACLK    0
#define TASSEL_ACLK     TASSEL0
#define TASSEL_SMCLK    TASSEL1
#define TASSEL_INCLK    TASSEL0 | TASSEL1
#endif

#ifndef TASSEL_DIV0
#define TASSEL_DIV0     0
#define TASSEL_DIV2     ID0 
#define TASSEL_DIV4     ID1
#define TASSEL_DIV8     ID0 | ID1
#endif

#ifndef TAMC_STOP
#define TAMC_STOP       0
#define TAMC_UP_CCR0    MC0
#define TAMC_UP         MC1
#define TAMC_UP_DOWN    MC0 | MC1
#endif

#if defined(__IAR_SYSTEMS_ICC__) && __VER__ < 300
#define TA0CTL          TACTL
#define TA0CCR2         TACCR2
#define TA0CCTL2        TACCTL2

#define TB0CTL          TBCTL
#define TB0CCR2         TBCCR2
#define TB0CCTL2        TBCCTL2
#endif

// UCTL - UART Control register
#ifndef UCTL_SWRST
#define UCTL_SWRST   0x01
#define UCTL_MM      0x02 
#define UCTL_SYNC    0x04
#define UCTL_LISTEN  0x08
#define UCTL_CHAR    0x10
#define UCTL_SP      0x20
#define UCTL_PEV     0x40
#define UCTL_PENA    0x80
#endif

// UTCTL - Transmit Control Register
#ifndef UTCTL_TXEPT
#define UTCTL_TXEPT  0x01
#define UTCTL_WAKE   0x04
#define UTCTL_URXSE  0x08
#define UTCTL_SSEL0  0x10
#define UTCTL_SSEL1  0x20
#define UTCTL_CKPL   0x40
#endif

#ifndef UTCTL_SSEL_UCLK
#define UTCTL_SSEL_UCLK  0x00
#define UTCTL_SSEL_ACLK  UTCTL_SSEL0
#define UTCTL_SSEL_MCLK  UTCTL_SSEL1 | UTCTL_SSEL0
#endif

// URCTL - Receive Control Register
#ifndef URCTL_RXERR
#define URCTL_RXERR  0x01
#define URCTL_RXWake 0x02
#define URCTL_URXWIE 0x04
#define URCTL_URXEIE 0x08
#define URCTL_BRK    0x10
#define URCTL_OE     0x20
#define URCTL_PE     0x40
#define URCTL_FE     0x80
#endif

// IE1 -  Interrupt enable register
#ifndef URXIE
#define URXIE        0x40
#define UTXIE        0x80
#endif

#ifndef URXE0                          // > IAR C V126B, constants included in header                        
// ME1 - Module enable 1
#define URXE0        0x40
#define UTXE0        0x80
#endif

#ifndef TAIV_NONE
/* TA3IV Definitions */
#define TAIV_NONE           (0x0000)    /* No Interrupt pending */
#define TAIV_TACCR1         (0x0002)    /* TACCR1_CCIFG */
#define TAIV_TACCR2         (0x0004)    /* TACCR2_CCIFG */
#define TAIV_6              (0x0006)    /* Reserved */
#define TAIV_8              (0x0008)    /* Reserved */
#define TAIV_TAIFG          (0x000A)    /* TAIFG */
#endif

// TPUART
#if defined(MSP430F1xx) || defined(MSP430F2xx)
#define UTXD0        0x10
#define URXD0        0x20
#define UTXD1        0x40
#define URXD1        0x80
#endif

#if defined(MSP430F4xx)
#define UTXD0        0x10
#define URXD0        0x20
#define UTXD1        0x01
#define URXD1        0x02
#endif

//#define I2C_SetSCL(value)       {if ((value) == 0)  P1DIR |= 0x02; else P1DIR &= 0x0fd;}          
//#define I2C_SetSDA(value)       {if ((value) == 0)  P1DIR |= 0x01; else P1DIR &= 0x0fe;}

//#define I2C_GetSCL()            ((P1IN & 0x02) >> 1)
//#define I2C_GetSDA()            (P1IN & 0x01)

//-------
#if defined(MSP430F1xx) || defined(MSP430F4xx) 
#if UART == 0
#define TUI_CommSendByte(byte)   U0TXBUF = (byte);
#endif
#if UART == 1
#define TUI_CommSendByte(byte)   U1TXBUF = (byte);
#endif
#endif

#if defined(MSP430F2xx) || defined(MSP430F54xx)
#if UART == 0
#define TUI_CommSendByte(byte)   UCA0TXBUF = (byte);
#endif
#if UART == 1
#define TUI_CommSendByte(byte)   UCA1TXBUF = (byte);
#endif
#endif


//------------------------------------------------------------------
// clock definitions
//------------------------------------------------------------------
#ifndef XT1_FREQU
//  #define XT1_FREQU       8000000           // Oszillator frequence in Hz (4915200)
  #define XT1_FREQU       4915200           // Oszillator frequence in Hz (4915200)
  #define XT2_FREQU       0
  #define DCO_FREQU       0
#endif
#ifndef MCLK_FREQU
  #define MCLK_FREQU      XT1_FREQU
  #define ACLK_FREQU      XT1_FREQU
  #define SMCLK_FREQU     DCO_FREQU            

  #define U0BR0_const  ((MCLK_FREQU / 19200) % 256)
  #define U0BR1_const  ((MCLK_FREQU / 19200) / 256)
  #define U0MCTL_const 0
#endif

//------------------------------------------------------------------
// definitions for the timer system
//------------------------------------------------------------------
#define CCR2_const        (ACLK_FREQU / 500 )            // const for CCR for clock of 2 ms (input divider: 0)
#define TM_CLOCK_COUNT    CCR2_const 
#define TM_MS_TIMER_ADDER 2
#define TM_S_TIMER_ADDER  500

#ifndef MSP_TM_TIMERBLOCK
  #define MSP_TM_TIMERBLOCK 0
#endif

#ifdef COMPILE_C               // include only for C files
//------------------------------------------------------------------
// function declaration
//------------------------------------------------------------------
void MSP430_InitSave(void);

//------------------------------------------------------------------
// Debug
//------------------------------------------------------------------
extern void DB_InitMcStack(void);
#endif // COMPILE_C

