//------------------------------------------------------------------
// $Workfile: MSP430_serialCom.c $
// $Archive: /eib_stk/appl/A1008TA_KIMaip_Eval/src/MSP430_serialCom.c $
//
// Project:           common utilities
//
// Description:
//
// $Author: Thiere_m $
//
// Copyright Tapko Technologies GmbH
//
// $Date: 20.05.11 10:53 $
// $Revision: 2 $
//------------------------------------------------------------------
#include "project.h"
#include "eib_stk.h"
#include "app.h"
#include "taUtil_fifo.h"
#include "taUtil_serialCom.h"

#if defined(MSP430F1xx) || defined(MSP430F2xx)
#define UTXD0        0x10
#define URXD0        0x20
#define UTXD1        0x40
#define URXD1        0x80
#endif


// MSP430 specific part
#ifndef ACLK_FREQU
#error ACLK_FREQU not defined
#endif

#if LSC_uartsEnabled != LSC_ENABLE_UART0
  #error "Use only Uart0 for this Application"
#endif

#if ((LSC_uartsEnabled & LSC_ENABLE_UART0) != 0) && ((LSC_uartsEnabled & LSC_ENABLE_UART1) != 0)
  #error "This file supports only one UART"
#endif

#define PORT0   0
#define PORT1   1

#if (LSC_uartsEnabled & LSC_ENABLE_UART0) != 0
  #define  USED_PORT    PORT0
#endif

// constant values for baud rate register
// for 12 MHz
#if (ACLK_FREQU == 12000000)
const unsigned short LSC_MSP430_BR[] =
{
  (ACLK_FREQU / 1200),    // 1200
  (ACLK_FREQU / 2400),    // 2400
  (ACLK_FREQU / 4800),    // 4800
  (ACLK_FREQU / 9600),    // 9600
  (ACLK_FREQU / 19200),   // 19200
  (ACLK_FREQU / 38400),   // 38400
  (ACLK_FREQU / 57600),   // 57600
  (ACLK_FREQU / 115200),  // 115200
  (ACLK_FREQU / 230400),  // 230400
  (ACLK_FREQU / 460800)   // 460800
};

// constant values for modulation register
const unsigned char LSC_MSP430_MCTL[] =
{
  0x00,    // 1200
  0x00,    // 2400
  0x00,    // 4800
  0x00,    // 9600
  0x00,    // 19200
  0x00,    // 38400
  0x00,    // 57600
};
#endif // ACLK_FREQU == 12000000


#if defined(MSP430F2xx) 
//------------------------------------------------------------------
void LSC_HWConfigUart(const LSC_SerialConfig *settings)
{
  unsigned char uartCtl = 0;

  if (settings->portNr != USED_PORT) 
    return;

                                                 // calculate control register
  uartCtl |=  (settings->dataBits == LSC_data7) ? UC7BIT : 0;
  uartCtl |= (settings->stop == LSC_stop1) ? 0 : UCSPB;
  switch (settings->parity)
  {
    case LSC_parityEven:
      uartCtl |= UCPEN | UCPAR;
      break;

    case LSC_parityOdd:
      uartCtl |= UCPEN;
      break;

    default:
      break;
  }


#if ((LSC_uartsEnabled & LSC_ENABLE_UART0) != 0)
  UCA0CTL0 = uartCtl;            // async. 8E1
  UCA0CTL1 = UCSSEL0 | UCRXEIE | UCBRKIE;        // ACLK

  UCA0BR0 = LSC_MSP430_BR[settings->baudrate] % 256;
  UCA0BR1 = LSC_MSP430_BR[settings->baudrate] / 256;
  UCA0MCTL = LSC_MSP430_MCTL[settings->baudrate];

  P3SEL |= URXD0 | UTXD0;              // select RxD + TxD at port3

  IFG2 &= ~(UCA0RXIFG | UCA0TXIFG);
  IE2 |= UCA0TXIE | UCA0RXIE;         // interrupt enable
#endif
}
#endif



#if defined(MSP430F2xx)
#if ((LSC_uartsEnabled & LSC_ENABLE_UART0) != 0)
  #define LSC_UartCheckEror()     (UCA0STAT & (UCRXERR | UCOE))
  #define LSC_Uartoverflow        (UCOE)
  #define LSC_UartGetRcvByte()    (UCA0RXBUF)
  #define LSC_IntHandlerRx        USCIAB0RX_VECTOR
  #define LSC_IntHandlerTx        USCIAB0TX_VECTOR
  #define OE                      UCOE
  #define LSC_ResetTxInt()        IFG2 &= ~UCA0TXIFG
  #define LSC_CommSendByte(byte)  UCA0TXBUF = (byte);
#endif
#endif


//------------------------------------------------------------------
//------------------------------------------------------------------
void TUI_ComInit(void)
{
}

void LSC_HWTxByte(unsigned char portNr, unsigned char byte)
{
  // insert portHandling for multi-port-usage
    LSC_CommSendByte(byte);
}

//------------------------------------------------------------------

void LSC_SerialTxInt(void)
{
  LSC_ResetTxInt();
  LSC_TxInt(USED_PORT);
}


//------------------------------------------------------------------
void LSC_SerialRxInt(void)
{
  unsigned char uartState = LSC_UartCheckEror();
  unsigned char rxbyte;

  if (uartState)
  {
    rxbyte = LSC_UartGetRcvByte();       // fetch byte from UART to delete error
  }
  else
  {
    rxbyte = LSC_UartGetRcvByte();       // fetch byte from UART
    LSC_RxInt(0, rxbyte);
  }
}

/*
#define INTERRUPT_HANDLER  LSC_IntHandlerTx
#include <interrupt.h>
{
  if (IFG2 & UCA0TXIFG)
    LSC_TxInt(1);

  if (IFG2 & UCB0TXIFG)
    IFG2 &= ~UCB0TXIFG;// sollte nicht auftreten, da Interrupt nicht enabled
}
*/


#pragma vector=LSC_IntHandlerRx
__interrupt void ISR_UartA_Rx(void)           // Intrreupt-routine for RX-Interrupt
{
  if (IFG2 & UCA0RXIFG)
    LSC_SerialRxInt();
}












