//------------------------------------------------------------------
//
// Copyright (c) 1999 - 2009 Tapko Technologies GmbH
//
// PROPRIETARY RIGHTS of Tapko Technologies GmbH are involved
// in the subject matter of this material. All manufacturing,
// reproduction, use, and sales rights pertaining to this subject matter
// are governed by the license agreement. The recipient of this software
// implicitly accepts the terms of the license.
//
// This file is part of the Libraries of Tapko Technologies GmbH
// (e.g. TAPKO EIB/KNX Communication Stack)
//
//------------------------------------------------------------------
// $Workfile: TaUtil_BasicTimer.c $   
// $Archive: /EIB_Stk/system_15/common/TaUtil_BasicTimer.c $     

// $Author: Adler_k $         
// 
// The timer system is based on timer variables, which are updated in interrupt
// The check of the timer (calculation of time), is done in a cyclic part.
// The interrupt part in in the specific project.
// 
// $Date: 6.09.10 12:21 $ 
// $Revision: 8 $         
//------------------------------------------------------------------
#include "project.h"
#include "eib_stk.h"

#include "TaUtil_types.h"
#include "TaUtil_Timer.h"

#ifdef DEBUG_WIN32
  #include "time.h"
#endif

//------------------------------------------------------------------
// help variables
//------------------------------------------------------------------
volatile unsigned long  TM_counter1ms;           // counter 1 ms     
volatile unsigned long  TM_counter1s;            // counter 1 s 
volatile unsigned short TM_counter1s_help;       // help variable to generate 1s clock

//------------------------------------------------------------------
// Initialization of the timer 
//------------------------------------------------------------------
void TMB_Init(void)
{
  _DINT();
  TM_counter1ms = 0;                             // reset timer counter
  TM_counter1s = 0;
  TM_counter1s_help = 0;

  _EINT();
}

//------------------------------------------------------------------
// returns the value of free running timer
// input: mode (ms / s)
// return value: current time & 0x7fff
//------------------------------------------------------------------
_CRITICAL unsigned long TMB_GetTime(unsigned char mode)
{
  if ((mode & TM_MODE_1S) == 0)                 
    return TM_counter1ms;
  else
    return TM_counter1s; 
}

//------------------------------------------------------------------
_CRITICAL unsigned long TM_GetSysTime(void)
{
  return TM_counter1ms;
}

