//------------------------------------------------------------------
// $Workfile: TaUtil_Fifo.c $
// $Archive: /eib_stk/appl_tapko/ta_termba/src/TaUtil_Fifo.c $

// $Author: Adler_k $
//
// Copyright Tapko Technologies GmbH
//
// $Date: 20.10.09 10:36 $
// $Revision: 6 $
//------------------------------------------------------------------
#include "project.h"
#include "knx_target.h"
#include "TaUtil_types.h"
#include "TaUtil_fifo.h"

void FIFO_Init(FIFO_Sstate *fifo, char *bufferPtr, unsigned bufferSize)
{
  fifo->buffer = bufferPtr;
  fifo->size = bufferSize;
  fifo->writePtr = 0;
  fifo->readPtr  = 0;
}


//------------------------------------------------------------------
void FIFO_Clear(FIFO_Sstate *fifo)
{
  fifo->writePtr = 0;
  fifo->readPtr  = 0;
}

//------------------------------------------------------------------
unsigned FIFO_IncrementPtr(unsigned ptr, unsigned size)
{
  unsigned newPtr = ptr;

  newPtr++;
  if (newPtr == size)
    newPtr = 0;
  return newPtr;
}

//------------------------------------------------------------------
_BOOL FIFO_AddByte(FIFO_Sstate *fifo, char byte)
{
  unsigned newPtr;

  newPtr = FIFO_IncrementPtr(fifo->writePtr, fifo->size);
  if (newPtr == fifo->readPtr)
    return _FALSE;
  fifo->buffer[fifo->writePtr] = byte;
  fifo->writePtr = newPtr;
  return _TRUE;
}

//------------------------------------------------------------------
_BOOL FIFO_GetByte(FIFO_Sstate *fifo, char *byte)
{

  if (fifo->readPtr == fifo->writePtr)
    return _FALSE;
  *byte = fifo->buffer[fifo->readPtr];
  fifo->readPtr  = FIFO_IncrementPtr(fifo->readPtr, fifo->size);
  return _TRUE;
}

//------------------------------------------------------------------
_BOOL FIFO_IsEmpty(FIFO_Sstate *fifo)
{
  return (fifo->readPtr == fifo->writePtr);
}

//------------------------------------------------------------------
_BOOL FIFO_IsFull(FIFO_Sstate *fifo)
{
  return (fifo->readPtr == FIFO_IncrementPtr(fifo->writePtr, fifo->size));
}

//------------------------------------------------------------------
_CRITICAL unsigned FIFO_GetFillState(FIFO_Sstate *fifo)
{
  if (fifo->writePtr >= fifo->readPtr)
    return (fifo->writePtr - fifo->readPtr);
  else
    return (fifo->writePtr + fifo->size - fifo->readPtr);
}

//------------------------------------------------------------------
_BOOL FIFO_AddString(FIFO_Sstate *fifo, char *buffer, unsigned char size)
{
  unsigned freeBytes;
  unsigned char count;

  freeBytes = fifo->size - FIFO_GetFillState(fifo);
  if (freeBytes <= size)
    return _FALSE;

  for (count = 0; count < size; count++)
    FIFO_AddByte(fifo, buffer[count]);

  return _TRUE;
}

//------------------------------------------------------------------
_BOOL FIFO_GetString(FIFO_Sstate *fifo, char *buffer, unsigned char *size, unsigned char delimiter)
{
  unsigned count;
  unsigned bufferPtr;
  unsigned char found = 0;

  bufferPtr = fifo->readPtr;
  count = 0;

  if (fifo->readPtr == fifo->writePtr)
    return _FALSE;

  while (bufferPtr != fifo->writePtr && found == 0)
  {
    buffer[count] = fifo->buffer[bufferPtr];
    count ++;
    if (count == *size)
    {
      found = 1;
      buffer[count] = 0;
    }
    else
    {
      if (fifo->buffer[bufferPtr] == delimiter)
      {
        found = 1;
        buffer[count] = 0;
      }
    }
    bufferPtr = FIFO_IncrementPtr(bufferPtr, fifo->size);
  }

  if (found == 0)
    return _FALSE;

  fifo->readPtr = bufferPtr;
  *size = count;
  return _TRUE;
}
