//------------------------------------------------------------------
// $Workfile: TaUtil_serialCom.c $
// $Archive: /eib_stk/appl_tapko/ta_termba/src/TaUtil_serialCom.c $
//
// Project:           common utilities
//
// Description:
//
// $Author: Adler_k $
//
// Copyright Tapko Technologies GmbH
//
// $Date: 11.12.09 12:27 $
// $Revision: 10 $
//------------------------------------------------------------------
#include "project.h"
#include "knx_target.h"
#include "TaUtil_serialCom.h"
//#include "msp430_serialCom.h"

LSC_SerialData *LSC_struct[LSC_noOfUarts];

//------------------------------------------------------------------
void LSC_Init(void)
{
  unsigned char count;

  for (count = 0; count < LSC_noOfUarts; count++)
    LSC_struct[count] = 0;
}

//------------------------------------------------------------------
void LSC_ConfigUart(const LSC_SerialConfig *config, LSC_SerialData *data)
{
  if (((LSC_uartsEnabled & (0x01 << config->portNr)) == 0))
    return;
  LSC_struct[config->portNr] = data;

  LSC_HWConfigUart(config);

  FIFO_Init(&data->rxFifo,  config->rxBuffer,  config->rxBufferSize);
  FIFO_Init(&data->txFifo, config->txBuffer,  config->txBufferSize);
  data->rxOverflow = 0;
  data->flags = config->flags;
}

//------------------------------------------------------------------
void LSC_TxInt(unsigned char portNr)
{
  LSC_SerialData *settings;
  char byte;

  settings = LSC_struct[portNr];

  if (FIFO_GetByte(&settings->txFifo, &byte))
  {
    settings->transmitting = 1;
    LSC_HWTxByte(portNr, byte);
  }
  else
  {
    settings->transmitting = 0;
  }
  LSC_SetTxLED();
}

//------------------------------------------------------------------
void LSC_RxInt(unsigned char portNr, unsigned char byte)
{
  LSC_SerialData *settings;

  settings = LSC_struct[portNr];
  if (!FIFO_AddByte(&settings->rxFifo, byte))
    settings->rxOverflow = 1;
  LSC_SetRxLED();
}

//------------------------------------------------------------------
_CRITICAL void LSC_StartTransmission(unsigned char portNr)
{
  if (LSC_struct[portNr]->transmitting == 0)
    LSC_TxInt(portNr);
}

//------------------------------------------------------------------
_BOOL LSC_TxByteIntern(unsigned char portNr, unsigned char byte)
{
  LSC_SerialData *settings;

  settings = LSC_struct[portNr];
  if (!FIFO_AddByte(&settings->txFifo, byte))
    return _FALSE;
  LSC_StartTransmission(portNr);
  return _TRUE;
}

//------------------------------------------------------------------
_BOOL LSC_TxByte(unsigned char portNr, unsigned char byte, unsigned char flags)
{
  LSC_SerialData *settings;
  _BOOL result;

  settings = LSC_struct[portNr];
  do
  {
    result = LSC_TxByteIntern(portNr, byte);
    WDG_RESET();
  }
  while (((flags | settings->flags) & LSC_TX_BLOCKING) & result != _TRUE);
  return result;
}

//------------------------------------------------------------------
_BOOL LSC_TxStringIntern(unsigned char portNr, char *data, unsigned char size)
{
  LSC_SerialData *settings;

  settings = LSC_struct[portNr];
  if (!FIFO_AddString(&settings->txFifo, data, size))
    return _FALSE;

  LSC_StartTransmission(portNr);
  return _TRUE;
}

//------------------------------------------------------------------
_BOOL LSC_TxString(unsigned char portNr, char *data, unsigned char size, unsigned char flags)
{
  LSC_SerialData *settings;
  _BOOL result;

  settings = LSC_struct[portNr];
  do
  {
    result = LSC_TxStringIntern(portNr, data, size);
  }
  while (((flags | settings->flags) & LSC_TX_BLOCKING) & result != _TRUE);
  return result;
}

_BOOL LSC_TxSpaceFree(unsigned char portNr, unsigned bytes2Write)
{
  LSC_SerialData *settings;

  settings = LSC_struct[portNr];
  if ((settings->txFifo.size - FIFO_GetFillState(&settings->txFifo)) <= bytes2Write)
    return _FALSE;

  return _TRUE;
}


//------------------------------------------------------------------
_BOOL LSC_RxByte(unsigned char portNr, char *byte, unsigned char flags)
{
  flags++;                                                 // eliminate compiler warning
  return FIFO_GetByte(&LSC_struct[portNr]->rxFifo, byte);
}

//------------------------------------------------------------------
_BOOL LSC_RxString(unsigned char portNr, char *data, unsigned char *size, unsigned char delimiter, unsigned char flags)
{
  LSC_SerialData *settings;

  flags++;                                                 // eliminate compiler warning
  settings = LSC_struct[portNr];
  return FIFO_GetString(&settings->rxFifo, data, size, delimiter);
}

//------------------------------------------------------------------
// ??????????????????
_BOOL LSC_RxLine(unsigned char portNr, char *data)
{
  unsigned char size = LSC_MAX_RX_LINE_LENGTH;

  return LSC_RxString(portNr, data, &size, '\r', 0);
}
