//------------------------------------------------------------------
//
// Copyright (c) 1999 - 2009 Tapko Technologies GmbH
//
// PROPRIETARY RIGHTS of Tapko Technologies GmbH are involved
// in the subject matter of this material. All manufacturing,
// reproduction, use, and sales rights pertaining to this subject matter
// are governed by the license agreement. The recipient of this software
// implicitly accepts the terms of the license.
//
// This file is part of the Libraries of Tapko Technologies GmbH
// (e.g. TAPKO EIB/KNX Communication Stack)
//
//------------------------------------------------------------------
// $Workfile: i2c_master.c $
// $Archive: /eib_stk/appl/A1008TA_KIMaip_Eval/src/i2c_master.c $
//
// $Author: Thiere_m $
//
// $Date: 20.05.11 22:24 $
// $Revision: 1 $
//------------------------------------------------------------------

#include "project.h"
#include "eib_stk.h"
#include "app.h"
#include "string.h"

#define IMM_UCB0

#ifdef IMM_UCB0
  #define IMM_UCBxCTL1  UCB0CTL1
  #define IMM_UCBxCTL0  UCB0CTL0
  #define IMM_UCBxBR0   UCB0BR0
  #define IMM_UCBxBR1   UCB0BR1
  #define IMM_UCBxI2COA UCB0I2COA
  #define IMM_UCBxI2CSA UCB0I2CSA
  #define IMM_UCBxI2CIE UCB0I2CIE
  #define IMM_UCBxTXBUF UCB0TXBUF
  #define IMM_UCBxRXBUF UCB0RXBUF
  #define IMM_UCBxSTAT UCB0STAT
  #define IMM_IE() UC0IE |= UCB0TXIE + UCB0RXIE
  #define IMM_TX_VECTOR USCIAB0TX_VECTOR
  #define IMM_RX_VECTOR USCIAB0RX_VECTOR
  #define IMM_TX_FLAG() (UC0IFG & UCB0TXIFG)
  #define IMM_RX_FLAG() (UC0IFG & UCB0RXIFG)
  #define IMM_TX_FLAG_Reset() (UC0IFG &= ~UCB0TXIFG)
  #define IMM_RX_FLAG_Reset() (UC0IFG &= ~UCB0RXIFG)
  #define IMM_SetPorts() {P3SEL |= BIT1 | BIT2;}   //P3.1,2
#endif

#define IMM_DefaultSlaveAddr      0x08
//------------------------------------------------------------------
enum IMM_EEvents
{
  IMM_ERxIsr         = 0x00,   // 
  IMM_ETxIsr         = 0x01,   // 
  IMM_ETimeout       = 0x02,   // 
  IMM_EDataInd       = 0x03,   // 
  IMM_EDataReq       = 0x04    // 
};


typedef enum 
{
  IMM_SIdle          = 0x00,   // 
  IMM_SRcv           = 0x01,   // 
  IMM_SSnd           = 0x02    // 
} IMM_tStates;

typedef enum 
{
  IMM_ANone          = 0x00,   // 
  IMM_AClearRxInt    = 0x01,   // 
  IMM_AClearTxInt    = 0x02,   // 
  IMM_ARcvByte       = 0x03,   // 
  IMM_ACancelRcv     = 0x04,   // 
  IMM_ASndByte       = 0x05,   // 
  IMM_ACancelSnd     = 0x06,   // 
  IMM_AStartRcv      = 0x07,   // 
  IMM_AStartSend     = 0x08    // 
} IMM_tActions;

struct IMM_SStateMachineEntry
{
  IMM_tStates nextState;
  IMM_tActions action;  
};

const struct IMM_SStateMachineEntry IMM_stateTable[5][3] = {
//States:   IMM_SIdle                        IMM_SRcv                         IMM_SSnd                             Events   
         {{ IMM_SIdle , IMM_AClearRxInt } , {IMM_SRcv  , IMM_ARcvByte    } , {IMM_SSnd  , IMM_AClearRxInt  }}, // IMM_ERxIsr
         {{ IMM_SIdle , IMM_AClearTxInt } , {IMM_SRcv  , IMM_AClearTxInt } , {IMM_SSnd  , IMM_ASndByte     }}, // IMM_ETxIsr
         {{ IMM_SIdle , IMM_ANone       } , {IMM_SIdle , IMM_ACancelRcv  } , {IMM_SIdle , IMM_ACancelSnd   }}, // IMM_ETimeout
         {{ IMM_SRcv  , IMM_AStartRcv   } , {IMM_SRcv  , IMM_ANone       } , {IMM_SSnd  , IMM_ANone        }}, // IMM_EDataInd
         {{ IMM_SSnd  , IMM_AStartSend  } , {IMM_SRcv  , IMM_ANone       } , {IMM_SSnd  , IMM_ANone        }}  // IMM_EDataReq
};

//------------------------------------------------------------------
typedef struct  {
  IMM_tStates state;
  unsigned char msg[MS_MSG_LENGTH];
  unsigned short byteCount;
} IMM_tData;

//------------------------------------------------------------------
IMM_tData IMM_data;
void IMM_Init(void)
{
  IMM_data.state = IMM_SIdle;
  IMM_data.byteCount = 0;

  IMM_SetPorts();                           // Assign I2C pins to USCI_B0
  IMM_UCBxCTL1 |= UCSWRST;                  // Enable SW reset

  IMM_UCBxCTL0 = UCMST + UCMODE_3 + UCSYNC;       // I2C Master, synchronous mode
  IMM_UCBxCTL1 = UCSSEL_1 + UCSWRST;              // Use ACLK, keep SW reset
  IMM_UCBxBR0 = 0x30;                         // set prescaler
  IMM_UCBxBR1 = 0;
  IMM_UCBxI2CSA = 0x2;                              // Set slave address
  IMM_UCBxCTL1 &= ~UCSWRST;                       // Clear SW reset, resume operation
  IMM_IE();
}

_CRITICAL void IMM_HandleStateMachine(unsigned char event, MSGPTR msg)
{
  unsigned char action;

                                   // calculate new state and action
  action = IMM_stateTable[event][IMM_data.state].action;
  IMM_data.state = IMM_stateTable[event][IMM_data.state].nextState;

  switch (action)
  {
    case IMM_AClearRxInt :
        IMM_TX_FLAG_Reset();
      break;
  
    case IMM_AClearTxInt :
        IMM_RX_FLAG_Reset();
      break;
  
    case IMM_ARcvByte :
        if (IMM_data.byteCount > 0)
        {
          if ((IMM_data.byteCount == IMM_data.msg[0]-1)
              || (IMM_data.msg[0] == 0)
              || (IMM_data.msg[0] == 1))
          {
            IMM_UCBxCTL1 |= UCTXSTP;                      // send stop condition
          }
        }
        IMM_data.msg[IMM_data.byteCount] = IMM_UCBxRXBUF;
        IMM_data.byteCount ++;
        if (IMM_data.byteCount > IMM_data.msg[0])
        {
          IMM_TX_FLAG_Reset();
          IMM_data.msg[MS_MSG_STATE]=KNX_TRUE;
          IMM_data.state = IMM_SIdle;
          TM_StopTimer(TM_IIC_TIMEOUT);
        }
      break;
  
    case IMM_ASndByte :
        if (IMM_data.byteCount <= (IMM_data.msg[0]))
        {
          IMM_UCBxTXBUF = IMM_data.msg[IMM_data.byteCount];
          IMM_data.byteCount ++;
        }
        else
        {
          IMM_UCBxCTL1 |= UCTXSTP;                      // send stop condition
          IMM_TX_FLAG_Reset();
          IMM_data.state = IMM_SIdle;
        }
      break;
  
    case IMM_ACancelRcv :
    case IMM_ACancelSnd :
        IMM_UCBxCTL1 |= UCTXSTP;                      // send stop condition
        IMM_data.msg[MS_MSG_STATE]=KNX_FALSE;
      break;
  
    case IMM_AStartRcv :
        if (    ((IMM_UCBxSTAT & UCBBUSY) != 0) 
             || ( IMM_data.msg[MS_MSG_STATE] != TA_FALSE)) 
        {
          IMM_data.state = IMM_SIdle;
        }
        else
        {
          IMM_data.byteCount = 0;
          
          IMM_UCBxI2CSA = IMM_DefaultSlaveAddr;        // Set slave address
          IMM_UCBxCTL1 &= ~UCTR;                             // configure as receiver
          IMM_UCBxCTL1 |= UCTXSTT;                      // send start condition
        }
        TM_StartTimer(TM_IIC_TIMEOUT, 500, 0);
      break;
  
    case IMM_AStartSend :
        if (IMM_data.msg[MS_MSG_STATE] != TA_FALSE)
        {
          IMM_data.state = IMM_SIdle;
        }
        else
        {
          UT_CopyMem(msg, IMM_data.msg, msg[0] + 1);
          msg[MS_MSG_STATE] = TA_FALSE;                 // reset the send-flag of the message

          IMM_data.byteCount = 0;
          
          IMM_UCBxI2CSA = IMM_DefaultSlaveAddr;                              // Set slave address
          IMM_UCBxCTL1 &= ~UCTXSTP;
          IMM_UCBxCTL1 |= UCTR;                             // configure as transmitter
          IMM_UCBxCTL1 |= UCTXSTT;                      // send start condition
          TM_StartTimer(TM_IIC_TIMEOUT, 500, 0);
        }
      break;
  
    case IMM_ANone :
    default:
      break;
  }
}

//------------------------------------------------------------------
#pragma vector = IMM_TX_VECTOR
__interrupt void IMM_DataIsr(void)
{
  if (IMM_TX_FLAG() != 0)         // TX interrupt
    IMM_HandleStateMachine(IMM_ETxIsr, 0);
  if (IMM_RX_FLAG() != 0)         // RX interrupt
    IMM_HandleStateMachine(IMM_ERxIsr, 0);

  if (IFG2 & UCA0TXIFG)
  {
    LSC_TxInt(0);
    IFG2 &= ~UCA0TXIFG;
  }
}


void IMM_Cycle(void)
{
  if ((P3IN & BIT0) != 0)
    IMM_HandleStateMachine(IMM_EDataInd, 0);

  if (TM_GetState(TM_IIC_TIMEOUT))
    IMM_HandleStateMachine(IMM_ETimeout,0);

  
  if (IMM_data.msg[MS_MSG_STATE])
  {
    IMD_MsgInd(IMM_data.msg);
    IMM_data.msg[MS_MSG_STATE] = TA_FALSE;
  }
}

void IMM_SendMessage(MSGPTR msg)
{
  IMM_HandleStateMachine(IMM_EDataReq, msg);
}

