//------------------------------------------------------------------
//
// Copyright (c) 1999 - 2009 Tapko Technologies GmbH
//
// PROPRIETARY RIGHTS of Tapko Technologies GmbH are involved
// in the subject matter of this material. All manufacturing,
// reproduction, use, and sales rights pertaining to this subject matter
// are governed by the license agreement. The recipient of this software
// implicitly accepts the terms of the license.
//
// This file is part of the Libraries of Tapko Technologies GmbH
// (e.g. TAPKO EIB/KNX Communication Stack)
//
//------------------------------------------------------------------
// $Workfile: i2c_msgHandler.c $
// $Archive: /eib_stk/appl/A1008TA_KIMaip_Eval/src/i2c_msgHandler.c $
//
// $Author: Thiere_m $
//
// $Date: 20.05.11 10:53 $
// $Revision: 2 $
//------------------------------------------------------------------

#include "project.h"
#include "eib_stk.h"
#include "app.h"

#define IMD_Mode0Enabled()     ((P2IN & BIT1) == 0)
#define IMD_Mode1Enabled()     ((P2IN & BIT0) == 0)

#define IMD_ModeDevice()            IMD_Mode1Enabled()
#define IMD_ModeSerialInterface()   IMD_Mode0Enabled()
#define IMD_ModeDisabled()          (!IMD_Mode0Enabled() && !IMD_Mode1Enabled())



#define IMD_HandleLed1(state)     {if (state) { P2OUT |= BIT2;} else { P2OUT &= ~BIT2;}}
#define IMD_HandleLed2(state)     {if (state) { P2OUT |= BIT3;} else { P2OUT &= ~BIT3;}}

#define IMD_HwGetKey()            ((P2IN & BIT4) != 0)
unsigned char IMD_lastKeyState;


#define IMD_NoOfObjects             3     // in this application we support 3 Objects
#define IMD_OBJECT_LED1             0
#define IMD_OBJECT_LED2             1
#define IMD_OBJECT_KEY              2

unsigned char IMD_objValue[IMD_NoOfObjects];
unsigned char IMD_objMsg[MS_MSG_LENGTH];

#define  OBJ_SendToIic                  IMD_objMsg[MS_MSG_STATE]  

// some vars to store the Paramter from the KIMaip
unsigned char IMD_PAR_addValue;
unsigned char IMD_PAR_sendValue;

#define KII_MsgGetService(msg)              ((msg)[1])
#define KII_MsgSetService(msg, service)     ((msg)[1] = service)
#define KII_GetObjectNo(msg)                GetInteger((unsigned short *) &msg[2])
#define KII_SetObjectNo(msg, index)         SetInteger((unsigned short *) &msg[2], index)
#define KII_MsgGetLength(msg)               ((msg)[0])
#define KII_MsgSetLength(msg, length)       ((msg)[0] = (length))
#define KII_GetObjectValue(msg)             (msg[4])
#define KII_SetObjectValue(msg, value)      (msg[4] = (value))
#define KII_GetSystemEventID(msg)           GetInteger((unsigned short *) &msg[2])
#define KII_GetSystemEventData(msg)         GetInteger((unsigned short *) &msg[4])

#define KII_SetParameterAddress(msg, startIndex)  SetInteger((unsigned short *) &msg[2], (startIndex))
#define KII_GetParameterAddress(msg)              GetInteger((unsigned short *) &msg[2])
#define KII_SetParameterCount(msg, cnt)           (msg[4] = (cnt))
#define KII_GetParameterCount(msg)                (msg[4])
#define KII_GetParameterValue(msg, index)         (msg[5 + (index)])


//--- commands forI2C ------------------------------
#define IIC_CMD_ValueWriteSend          0x10
#define IIC_CMD_ValueReadSend           0x11
#define IIC_CMD_ValueResponseSend       0x12

#define IIC_CMD_ValueReceive            0x20
#define IIC_CMD_ValueReadReceive        0x21
#define IIC_CMD_ValueReceivePack        0x28
#define IIC_CMD_ValueReadReceivePack    0x29

#define IIC_CMD_ObjDescrWrite           0x30 
#define IIC_CMD_ObjDescrRead            0x31 
#define IIC_CMD_ObjDescrResponse        0x32 

#define IIC_CMD_ObjGroupAddrAddSend     0x34
#define IIC_CMD_ObjGroupAddrAdd         0x35
#define IIC_CMD_ObjGroupAddrDelete      0x36
#define IIC_CMD_ObjGroupAddrDeleteAll   0x37
#define IIC_CMD_ObjGroupAddrRead        0x38
#define IIC_CMD_ObjGroupAddrResponse    0x39

#define IIC_CMD_ParameterWrite          0x40
#define IIC_CMD_ParameterRead           0x41
#define IIC_CMD_ParameterResponse       0x42

#define IIC_CMD_InterfaceObjectWrite    0x50
#define IIC_CMD_InterfaceObjectRead     0x51
#define IIC_CMD_InterfaceObjectResponse 0x52

#define IIC_CMD_SystemParameterWrite    0x60
#define IIC_CMD_SystemParameterRead     0x61
#define IIC_CMD_SystemParameterResponse 0x62

#define IIC_CMD_TranspGroupWriteSend    0x70
#define IIC_CMD_TranspGroupReadSend     0x71
#define IIC_CMD_TranspGroupRespSend     0x72
#define IIC_CMD_TranspRcvMask           BIT2
#define IIC_CMD_TranspConMask           BIT3
#define IIC_CMD_TranspNegConMask        (BIT3 + BIT2)
#define IIC_CMD_TranspGroupWriteRcv     (IIC_CMD_TranspGroupWriteSend + IIC_CMD_TranspRcvMask)
#define IIC_CMD_TranspGroupReadRcv      (IIC_CMD_TranspGroupReadSend  + IIC_CMD_TranspRcvMask)
#define IIC_CMD_TranspGroupRespRcv      (IIC_CMD_TranspGroupRespSend  + IIC_CMD_TranspRcvMask)
#define IIC_CMD_TranspGroupWriteCon     (IIC_CMD_TranspGroupWriteSend + IIC_CMD_TranspConMask)
#define IIC_CMD_TranspGroupReadCon      (IIC_CMD_TranspGroupReadSend  + IIC_CMD_TranspConMask)
#define IIC_CMD_TranspGroupRespCon      (IIC_CMD_TranspGroupRespSend  + IIC_CMD_TranspConMask)
#define IIC_CMD_TranspGroupWriteNegCon  (IIC_CMD_TranspGroupWriteSend + IIC_CMD_TranspNegConMask)
#define IIC_CMD_TranspGroupReadNegCon   (IIC_CMD_TranspGroupReadSend  + IIC_CMD_TranspNegConMask)
#define IIC_CMD_TranspGroupRespNegCon   (IIC_CMD_TranspGroupRespSend  + IIC_CMD_TranspNegConMask)

#define IIC_CMD_SysEvent                0x80

#define IIC_CMD_ResetToDefault          0xD0

#define IIC_CMD_ErrorInd                0xE0

//-------------------------------------------------------------------
// helper function to read an integer value 
// BYTE order
//-------------------------------------------------------------------
unsigned short GetInteger(unsigned short * ptr)         
{
  return (*((unsigned char *) ptr) <<8) + *(((unsigned char *)ptr) + 1);
}

//-------------------------------------------------------------------
// helper function to write an integer value 
// BYTE order
//-------------------------------------------------------------------
void SetInteger(unsigned short *ptr, unsigned short value)  
{
  *(unsigned char *)ptr = _HIBYTE(value);
  *((unsigned char *)ptr+1) = _LOBYTE(value);
}  

// Creates a message to IIC with fixed datasize (=1)
void IMD_CreateValueWrite(unsigned short objNo, unsigned char value)
{
  KII_MsgSetLength(IMD_objMsg, 4);          // Length for ValueWrite with one data-byte
  KII_MsgSetService (IMD_objMsg, IIC_CMD_ValueWriteSend);
  KII_SetObjectNo(IMD_objMsg, objNo);
  KII_SetObjectValue(IMD_objMsg, value);
  OBJ_SendToIic = KNX_TRUE;
}


// Creates a message to IIC with fixed datasize (=1)
void IMD_CreateValueResponse(unsigned short objNo, unsigned char value)
{
  KII_MsgSetLength(IMD_objMsg, 4);          // Length for ValueResponse with one data-byte
  KII_MsgSetService (IMD_objMsg, IIC_CMD_ValueResponseSend);
  KII_SetObjectNo(IMD_objMsg, objNo);
  KII_SetObjectValue(IMD_objMsg, value);
  OBJ_SendToIic = KNX_TRUE;
}

// Creates a message to IIC with fixed datasize (=1)
void IMD_CreateGetParamter(void)
{
  KII_MsgSetLength(IMD_objMsg, 4);          // Length for ParamterRead
  KII_MsgSetService (IMD_objMsg, IIC_CMD_ParameterRead);
  KII_SetParameterAddress(IMD_objMsg, 0);   // the parameter for this application are located at address 0
  KII_SetParameterCount(IMD_objMsg, 2);     // we want to read two parameter
  OBJ_SendToIic = KNX_TRUE;
}


void IMD_HandleKey(void)
{
  if (IMD_HwGetKey() && (IMD_lastKeyState==0))
  {
    IMD_objValue[IMD_OBJECT_KEY] += IMD_PAR_addValue;      // add the requested value to the object
    if (IMD_PAR_sendValue != 0)           
      IMD_CreateValueWrite(IMD_OBJECT_KEY, IMD_objValue[IMD_OBJECT_KEY]);
  }
  IMD_lastKeyState = IMD_HwGetKey();
}


void IMD_MsgInd(unsigned char *msg)
{
  if (IMD_ModeSerialInterface())
    PrintTelegram(msg, '>');
  
  if (IMD_ModeDevice())
  {
    switch (KII_MsgGetService(msg))
    {
      case IIC_CMD_ValueReceive:
          if (KII_GetObjectNo(msg) < IMD_NoOfObjects)       // check if we will support this object
            IMD_objValue[KII_GetObjectNo(msg)] = KII_GetObjectValue(msg);
        break;

      case IIC_CMD_ValueReadReceive:
          if (KII_GetObjectNo(msg) < IMD_NoOfObjects)       // check if we will support this object
            IMD_CreateValueResponse(KII_GetObjectNo(msg), IMD_objValue[KII_GetObjectNo(msg)]);
        break;
        
      case IIC_CMD_ParameterResponse:
          if((KII_GetParameterAddress(msg) == 0) && (KII_GetParameterCount(msg) == 2))
          {   
            // response is according to our request (Startindex = 0 and count = 2)
            IMD_PAR_sendValue = KII_GetParameterValue(msg, 0);
            IMD_PAR_addValue  = KII_GetParameterValue(msg, 1);
          }
        break;
      case IIC_CMD_SysEvent:
          if (KII_GetSystemEventID(msg)==0x0100 && KII_GetSystemEventData(msg) == 0x0300)   
          {
            // Event for Application started 
            // that means, that the KIMaip has done a reset
            // start to read the Parameter
            IMD_CreateGetParamter();
          }
        break;
        
      case IIC_CMD_ErrorInd:    
          // TODO    Handle Error-Messages
        break;
     
    }
    
  };

}

void IMD_HandleLed(void)
{
  if (IMD_ModeDisabled())
  {
    IMD_HandleLed1((TM_counter1s % 2) == 0);
    IMD_HandleLed2((TM_counter1s % 2) != 0);
  } 
  else if (!IMD_ModeDevice() && IMD_ModeSerialInterface())
  { 
    IMD_HandleLed1((TM_counter1s % 2) == 0);
    IMD_HandleLed2(TA_FALSE);
  }
  else  if (IMD_ModeDevice())
  {
    IMD_HandleLed1(IMD_objValue[IMD_OBJECT_LED1]);
    IMD_HandleLed2(IMD_objValue[IMD_OBJECT_LED2]);
    IMD_HandleKey();
  }
  
}


void IMD_Init(void)
{
  IMM_Init();
  
  P2DIR |= BIT2 | BIT3;
  P2DIR &= ~(BIT0 | BIT1);
  
  IMD_objValue[0]=0;        // init all object values
  IMD_objValue[1]=0;
  IMD_objValue[2]=0;

  IMD_HandleLed1(TA_TRUE);
  IMD_HandleLed2(TA_TRUE);
  IMD_lastKeyState = IMD_HwGetKey();

  IMD_CreateGetParamter();  // read Parameter from KIMaip after startup
}


void IMD_Cycle(void)
{
  
  IMD_HandleLed();
  
  if (SER_SendToIic && IMD_ModeSerialInterface())
  {                                   // message should be sended to IC
    IMM_SendMessage(SER_iicMsgRcv);

    if (!SER_SendToIic)               // send-flag is cleared? -> message was sended
      PrintTelegram(SER_iicMsgRcv, '<');
  }
    

  if (OBJ_SendToIic && IMD_ModeDevice())    
  {                                   // message should be sended to IC
    IMM_SendMessage(IMD_objMsg);

    if (!OBJ_SendToIic)               // send-flag is cleared? -> message was sended
      PrintTelegram(IMD_objMsg, '<');
  }
  
  IMM_Cycle();
}




