//------------------------------------------------------------------
// $Workfile: serObjectInterface.c $
// $Archive: /eib_stk/appl/A1008TA_KIMaip_Eval/src/serObjectInterface.c $

// $Author: Thiere_m $
//
// Copyright Tapko Technologies GmbH
//
// $Date: 20.05.11 10:53 $
// $Revision: 3 $
//------------------------------------------------------------------
// offen:

#include "project.h"
#include "eib_stk.h"
#include "app.h"
#include "TaUtil_serialCom.h"
#include "string.h"


void PrintResponseEnd(void);

//------------------------------------------------------------------
// RAM variables
//------------------------------------------------------------------
unsigned char SER_iicMsgRcv[MS_MSG_LENGTH];

const unsigned char SER_hexChar[] = "0123456789ABCDEF";


//------------------------------------------------------------------
//******************************************************************
// functions to print responses
//******************************************************************
//------------------------------------------------------------------

//------------------------------------------------------------------
// PrintResponseEnd
// prints the end of a response
//------------------------------------------------------------------
void PrintResponseEnd(void)
{
  LSC_TxString(SER_PORT, "\n\r", 2, 0);                    // new line
}


//------------------------------------------------------------------
// PrintHexValueWithoutSpace
// prints one hex value
// inputs: value: value to be print
//         bytes: number of characters to be print
//------------------------------------------------------------------
void PrintHexValue(unsigned long value, char bytes)
{
  bytes *= 2;
  for (; bytes > 0; bytes--)                               // transmit number of nibbles
    LSC_TxByte (SER_PORT, SER_hexChar[(value >> (4 * (bytes - 1))) & 0x0f], 0);
}


//------------------------------------------------------------------
// PrintTelegram
// prints a string with variable length, length is first byte of telegram
// inputs: l_msg: pointer to data
//         startSign: first sign, which is sended
//------------------------------------------------------------------
void PrintTelegram(unsigned char *l_msg, char startSign)
{
  unsigned char count = 0;

  LSC_TxByte (SER_PORT, startSign, 0);
  LSC_TxByte (SER_PORT, ' ', 0);
  
  while (count <= l_msg[MSG_LENGTH])
  {
    PrintHexValue(l_msg[count],1);
    count ++;
  }
  PrintResponseEnd();
}



//------------------------------------------------------------------
//------------------------------------------------------------------

KNX_BOOL EleminateBS(char rcvByte)
{
  if ((rcvByte == 0x08) && (SER_RcvPos > 0))
  {
    SER_RcvPos--;
  }
  return (rcvByte == 0x08);
}


KNX_BOOL SendToIic(char rcvByte)
{
  if ((rcvByte == '\r') && (SER_RcvPos > 0))
  {
    SER_SendToIic = TA_TRUE;
    SER_RcvPos = 0;
  }
  return (rcvByte == '\r');
}

typedef struct
{
  char          rxValue;
  unsigned char realValue;
} SER_SValidSigns;

SER_SValidSigns SER_salidSigns[]={
  {'0',0x00},
  {'1',0x01},
  {'2',0x02},
  {'3',0x03},
  {'4',0x04},
  {'5',0x05},
  {'6',0x06},
  {'7',0x07},
  {'8',0x08},
  {'9',0x09},
  {'a',0x0a},
  {'b',0x0b},
  {'c',0x0c},
  {'d',0x0d},
  {'e',0x0e},
  {'f',0x0f},
  {'A',0x0a},
  {'B',0x0b},
  {'C',0x0c},
  {'D',0x0d},
  {'E',0x0e},
  {'F',0x0f},
  {0,0} // Endekennung
};

KNX_BOOL GetValue(char *value)
{
  SER_SValidSigns *tablePtr = &SER_salidSigns[0];
  
  while ((tablePtr->rxValue!=0) && (tablePtr->rxValue != *value))
    tablePtr++;

  if (tablePtr->rxValue==0)
    return KNX_FALSE;

  *value = tablePtr->realValue;
  return KNX_TRUE;
}
//------------------------------------------------------------------
// main loop for the serial interface
//------------------------------------------------------------------
void SerAsciiInterface_Main(void)
{
  char rxByte;                                   // number of received bytes

  while (    (SER_SendToIic == TA_FALSE)
          && LSC_RxByte(SER_PORT, &rxByte, 0))
  {                                                        // string received
    if (!EleminateBS(rxByte) && !SendToIic(rxByte))
    {
      if (GetValue(&rxByte))
      {
        if ((SER_RcvPos % 2) == 0)
          SER_iicMsgRcv[SER_RcvPos/2] = 0;
        SER_iicMsgRcv[SER_RcvPos/2] |= rxByte << ((SER_RcvPos % 2) ? 0 : 4);
        if (SER_RcvPos < (2 * KNX_MAX_APDU_LENGTH))
          SER_RcvPos++;
      }
    }
  }
}

//------------------------------------------------------------------
// !!!!!!!!!!!!!!!!!!!!!!!!!!
//------------------------------------------------------------------
void SerAsciiInterface_Init(void)
{
  LSC_SerialConfig serConfig;

  memcpy(&serConfig, &com1_config, sizeof(serConfig));

  LSC_Init();
  LSC_ConfigUart(&serConfig, &com1_data);
  SER_SendToIic = TA_FALSE;
}
